@@ -19,3 +19,7 @@ LINE = re.compile(
     LINE_RE,
     re.MULTILINE,
 )
+
+ONE_LINER_RE = r"^\s*(\d+)\s*\n" + LINE_RE
+
+ONE_LINER = re.compile(ONE_LINER_RE, re.MULTILINE)
